import Loader from 'components/Loader/Loader'
import FluidPerformanceIndicator from 'components/PerformanceIndicator/FluidPerformanceIndicator'
import { useClient } from 'cozy-client'
import { FluidType } from 'enum/fluid.enum'
import { TimeStep } from 'enum/timeStep.enum'
import { DateTime } from 'luxon'
import { PerformanceIndicator } from 'models'
import React, { useEffect, useState } from 'react'
import { useSelector } from 'react-redux'
import ConsumptionService from 'services/consumption.service'
import ConfigService from 'services/fluidConfig.service'
import PerformanceIndicatorService from 'services/performanceIndicator.service'
import { AppStore } from 'store'
import AnalysisConsumption from './AnalysisConsumption'
import AnalysisErrorModal from './AnalysisErrorModal'
import ElecHalfHourMonthlyAnalysis from './ElecHalfHourMonthlyAnalysis'
import MaxConsumptionCard from './MaxConsumptionCard'
import './monthlyanalysis.scss'
import TotalAnalysisChart from './TotalAnalysisChart'

interface MonthlyAnalysisProps {
  analysisDate: DateTime
  saveLastScrollPosition: () => void
  scrollPosition: number
}

const MonthlyAnalysis: React.FC<MonthlyAnalysisProps> = ({
  analysisDate,
  saveLastScrollPosition,
  scrollPosition,
}: MonthlyAnalysisProps) => {
  const client = useClient()
  const { fluidTypes } = useSelector((state: AppStore) => state.ecolyo.global)
  const [performanceIndicators, setPerformanceIndicators] = useState<
    PerformanceIndicator[]
  >([])
  const [loadAnalysis, setLoadAnalysis] = useState<boolean>(false)
  const [aggregatedPerformanceIndicators, setAggregatedPerformanceIndicators] =
    useState<PerformanceIndicator>({
      value: 0,
      compareValue: 0,
      percentageVariation: 0,
    })
  const [isLoaded, setIsLoaded] = useState<boolean>(false)
  const configService = new ConfigService()
  const fluidConfig = configService.getFluidConfig()
  const timeStep = TimeStep.MONTH

  useEffect(() => {
    let subscribed = true
    async function populateData() {
      if (subscribed) {
        setIsLoaded(false)
        const consumptionService = new ConsumptionService(client)
        const performanceIndicatorService = new PerformanceIndicatorService()
        const periods = {
          timePeriod: {
            startDate: analysisDate.minus({ month: 1 }).startOf('month'),
            endDate: analysisDate.minus({ month: 1 }).endOf('month'),
          },
          comparisonTimePeriod: {
            startDate: analysisDate.minus({ month: 2 }).startOf('month'),
            endDate: analysisDate.minus({ month: 2 }).endOf('month'),
          },
        }
        const fetchedPerformanceIndicators =
          await consumptionService.getPerformanceIndicators(
            periods.timePeriod,
            timeStep,
            fluidTypes,
            periods.comparisonTimePeriod
          )
        if (subscribed) {
          if (fetchedPerformanceIndicators) {
            setPerformanceIndicators(fetchedPerformanceIndicators)
            setLoadAnalysis(false)
            for (let i = 0; i < fetchedPerformanceIndicators.length; i++) {
              if (fetchedPerformanceIndicators[i]?.value) {
                setLoadAnalysis(true)
              }
            }

            setAggregatedPerformanceIndicators(
              performanceIndicatorService.aggregatePerformanceIndicators(
                fetchedPerformanceIndicators
              )
            )
          }
          setIsLoaded(true)
        }
      }
    }
    populateData()

    return () => {
      saveLastScrollPosition()
      subscribed = false
    }
  }, [client, timeStep, fluidTypes, analysisDate, saveLastScrollPosition])

  useEffect(() => {
    if (isLoaded) {
      const app = document.querySelector('.app-content')
      window.scrollTo(0, scrollPosition)
      app?.scrollTo(0, scrollPosition)
    }
  }, [isLoaded, scrollPosition])

  return (
    <>
      {!isLoaded && (
        <div className="analysis-container-spinner" aria-busy="true">
          <Loader />
        </div>
      )}
      {isLoaded && (
        <div className="analysis-root black">
          {fluidTypes.length >= 1 ? (
            <>
              <div className="analysis-content">
                <div>
                  {fluidConfig.map(
                    fluid =>
                      fluidTypes.includes(fluid.fluidTypeId) && (
                        <FluidPerformanceIndicator
                          key={fluid.konnectorConfig.slug}
                          fluidType={fluid.fluidTypeId}
                          performanceIndicator={
                            performanceIndicators[fluid.fluidTypeId]
                          }
                          date={analysisDate
                            .minus({ month: 1 })
                            .startOf('month')}
                        />
                      )
                  )}
                </div>
              </div>

              {loadAnalysis && (
                <>
                  <div className="analysis-content">
                    <div className="card rich-card">
                      <TotalAnalysisChart
                        analysisDate={analysisDate}
                        fluidTypes={fluidTypes}
                      />
                    </div>
                  </div>
                  <div className="analysis-content">
                    <div className="card rich-card">
                      <MaxConsumptionCard analysisDate={analysisDate} />
                    </div>
                  </div>
                  <div className="analysis-content">
                    <div className="card rich-card">
                      <AnalysisConsumption
                        aggregatedPerformanceIndicator={
                          aggregatedPerformanceIndicators
                        }
                        performanceIndicators={performanceIndicators}
                        analysisDate={analysisDate}
                      />
                    </div>
                  </div>
                  {fluidTypes.includes(FluidType.ELECTRICITY) && (
                    <div className="analysis-content">
                      <div className="card">
                        <ElecHalfHourMonthlyAnalysis
                          analysisDate={analysisDate}
                          perfIndicator={
                            performanceIndicators[FluidType.ELECTRICITY]
                          }
                        />
                      </div>
                    </div>
                  )}
                </>
              )}
            </>
          ) : (
            <AnalysisErrorModal />
          )}
        </div>
      )}
    </>
  )
}

export default MonthlyAnalysis
