import StyledSwitch from 'components/CommonKit/Switch/StyledSwitch'
import HalfHourNoData from 'components/HalfHourNoData/HalfHourNoData'
import useExploration from 'components/Hooks/useExploration'
import TimeStepSelector from 'components/TimeStepSelector/TimeStepSelector'
import { useClient } from 'cozy-client'
import { useI18n } from 'cozy-ui/transpiled/react/I18n'
import { FluidType } from 'enum/fluid.enum'
import { TimeStep } from 'enum/timeStep.enum'
import { UsageEventType } from 'enum/usageEvent.enum'
import { UserExplorationID } from 'enum/userExploration.enum'
import React, { useEffect, useState } from 'react'
import { useSelector } from 'react-redux'
import ConsumptionService from 'services/consumption.service'
import UsageEventService from 'services/usageEvent.service'
import { AppStore } from 'store'
import './fluidChart.scss'
import FluidChartSwipe from './FluidChartSwipe'

interface FluidChartProps {
  fluidType: FluidType
  setActive: React.Dispatch<React.SetStateAction<boolean>>
}

const FluidChart: React.FC<FluidChartProps> = ({
  fluidType,
  setActive,
}: FluidChartProps) => {
  const { t } = useI18n()
  const client = useClient()
  const {
    chart: { currentTimeStep, selectedDate },
    global: { fluidStatus },
  } = useSelector((state: AppStore) => state.ecolyo)

  const [, setValidExploration] = useExploration()
  const [isLoaded, setIsLoaded] = useState<boolean>(false)
  const [showCompare, setShowCompare] = useState<boolean>(false)
  const [containsHalfHourData, setContainsHalfHourData] =
    useState<boolean>(false)

  const handleChangeSwitch = async () => {
    if (!showCompare) {
      await UsageEventService.addEvent(client, {
        type: UsageEventType.CONSUMPTION_COMPARE_EVENT,
        target: TimeStep[currentTimeStep].toLowerCase(),
        context: FluidType[fluidType].toLowerCase(),
      })
    }
    setShowCompare(!showCompare)
  }

  useEffect(() => {
    let subscribed = true
    async function loadData() {
      const consumptionService = new ConsumptionService(client)
      if (fluidType === FluidType.ELECTRICITY) {
        const halfHourData = await consumptionService.checkDoctypeEntries(
          FluidType.ELECTRICITY,
          TimeStep.HALF_AN_HOUR
        )
        if (halfHourData) {
          setContainsHalfHourData(true)
        }
      }
      if (subscribed) {
        setIsLoaded(true)
      }
    }

    loadData()

    return () => {
      subscribed = false
    }
  }, [client, fluidStatus, selectedDate, currentTimeStep, fluidType])

  useEffect(() => {
    if (containsHalfHourData && currentTimeStep === TimeStep.HALF_AN_HOUR) {
      setValidExploration(UserExplorationID.EXPLORATION004)
    } else if (currentTimeStep === TimeStep.YEAR) {
      setValidExploration(UserExplorationID.EXPLORATION003)
    }
  }, [containsHalfHourData, currentTimeStep, setValidExploration])

  const getCompareLabel = (currentTimeStep: TimeStep) => {
    return t(`timestep.${TimeStep[currentTimeStep].toLowerCase()}.comparelabel`)
  }

  return (
    <>
      {isLoaded && (
        <div className="fluidchart-root">
          {currentTimeStep === TimeStep.HALF_AN_HOUR &&
          !containsHalfHourData ? (
            <HalfHourNoData />
          ) : (
            <div className="fluidchart-content">
              <FluidChartSwipe
                fluidType={fluidType}
                showCompare={
                  currentTimeStep === TimeStep.YEAR ? false : showCompare
                }
                setActive={setActive}
              />
            </div>
          )}
          <TimeStepSelector fluidType={fluidType} />
          {currentTimeStep !== TimeStep.YEAR && (
            <div className="fluidchart-footer" onClick={handleChangeSwitch}>
              <div className="fluidchart-footer-compare text-15-normal">
                <StyledSwitch
                  fluidType={fluidType}
                  checked={showCompare}
                  inputProps={{
                    'aria-label': t(
                      'consumption.accessibility.checkbox_compare'
                    ),
                  }}
                />
                <span className="fluidchart-footer-label graph-switch-text">
                  {getCompareLabel(currentTimeStep)}
                </span>
              </div>
            </div>
          )}
        </div>
      )}
    </>
  )
}

export default FluidChart
