import IconButton from '@material-ui/core/IconButton'
import Header from 'components/Header/Header'
import { ScreenType } from 'enum/screen.enum'
import { mount } from 'enzyme'
import React from 'react'
import { Provider } from 'react-redux'
import configureStore from 'redux-mock-store'
import * as ModalAction from 'store/modal/modal.actions'
import { globalStateData } from '../../../tests/__mocks__/globalStateData.mock'

jest.mock('cozy-ui/transpiled/react/I18n', () => {
  return {
    useI18n: jest.fn(() => {
      return {
        t: (str: string) => str,
      }
    }),
  }
})

const mockedNavigate = jest.fn()
jest.mock('react-router-dom', () => ({
  ...jest.requireActual('react-router-dom'),
  useNavigate: () => mockedNavigate,
}))

const mockSetHeaderHeight = jest.fn()
const mockStore = configureStore([])

describe('Header component', () => {
  it('should be rendered correctly', () => {
    const store = mockStore({
      ecolyo: {
        global: globalStateData,
      },
    })
    const wrapper = mount(
      <Provider store={store}>
        <Header
          setHeaderHeight={mockSetHeaderHeight}
          desktopTitleKey="mockKey"
        />
      </Provider>
    )
    expect(wrapper.find('.header')).toHaveLength(1)
  })

  it('should display title and back button when desktopTitle key provided and displayBackArrow is true', () => {
    const store = mockStore({
      ecolyo: {
        global: globalStateData,
      },
    })
    const wrapper = mount(
      <Provider store={store}>
        <Header
          desktopTitleKey={'KEY'}
          displayBackArrow={true}
          setHeaderHeight={mockSetHeaderHeight}
        />
      </Provider>
    )
    expect(wrapper.find('.header-text-desktop').first().text()).toEqual('KEY')
    expect(
      wrapper.find(IconButton).find('.header-back-button').first()
    ).toHaveLength(1)
  })

  it('should call handleClickBack when back button is clicked', () => {
    const store = mockStore({
      ecolyo: {
        global: { ...globalStateData, screenType: ScreenType.DESKTOP },
      },
    })
    const wrapper = mount(
      <Provider store={store}>
        <Header
          desktopTitleKey={'KEY'}
          displayBackArrow={true}
          setHeaderHeight={mockSetHeaderHeight}
        />
      </Provider>
    )
    wrapper
      .find(IconButton)
      .find('.header-back-button')
      .first()
      .simulate('click')
    expect(mockedNavigate).toHaveBeenCalled()
  })

  it('should call handleClickFeedbacks when feedback button is clicked', () => {
    const store = mockStore({
      ecolyo: {
        global: globalStateData,
        modal: {
          isFeedbacksOpen: false,
        },
      },
    })
    const wrapper = mount(
      <Provider store={store}>
        <Header
          setHeaderHeight={mockSetHeaderHeight}
          desktopTitleKey="mockKey"
        />
      </Provider>
    )
    const updateModalSpy = jest.spyOn(ModalAction, 'openFeedbackModal')
    wrapper
      .find(IconButton)
      .find('.header-feedbacks-button')
      .first()
      .simulate('click')
    expect(updateModalSpy).toHaveBeenCalledWith(true)
  })
})
