import { Client, QueryResult } from 'cozy-client'
import { PROFILE_DOCTYPE } from 'doctypes'
import { Profile } from 'models'
import { Notes } from 'models/releaseNotes.model'
import { Schema } from 'models/schema.models'
import mockClient from '../../tests/__mocks__/client'
import * as Migrate from './migration'
import { MIGRATION_RESULT_FAILED } from './migration.data'
import { MigrationService } from './migration.service'
import { Migration, MigrationResult } from './migration.type'

const migrateSpy = jest.spyOn(Migrate, 'migrate')
describe('Migration service', () => {
  const ms = new MigrationService(mockClient, jest.fn())
  const releaseNotes: Notes = {
    title: '',
    description: '',
  }
  beforeEach(() => {
    migrateSpy.mockClear()
  })
  it('should run migrations', async () => {
    const schema: Schema = { _id: '1', version: 0 }
    const mockQueryResult: QueryResult<Schema[]> = {
      data: [schema],
      bookmark: '',
      next: false,
      skip: 0,
    }
    const migrations: Migration[] = [
      {
        baseSchemaVersion: 0,
        targetSchemaVersion: 1,
        appVersion: '1.2.4',
        description: 'Removing mailToken from profil',
        docTypes: PROFILE_DOCTYPE,
        releaseNotes: releaseNotes,
        run: async (client: Client, docs: any[]): Promise<Profile[]> => {
          return docs.map(doc => {
            if (doc.mailToken) {
              delete doc.mailToken
            }
            return doc
          })
        },
      },
    ]
    mockClient.query.mockResolvedValue(mockQueryResult)
    await ms.runMigrations(migrations)
    expect(migrateSpy).toBeCalledTimes(1)
  })

  it('should run migrations with one fail', async () => {
    const schema: Schema = { _id: '1', version: 0 }
    const mockQueryResult: QueryResult<Schema[]> = {
      data: [schema],
      bookmark: '',
      next: false,
      skip: 0,
    }
    const migrations: Migration[] = [
      {
        baseSchemaVersion: 0,
        targetSchemaVersion: 1,
        appVersion: '1.2.4',
        description: 'Removing mailToken from profil',
        docTypes: PROFILE_DOCTYPE,
        releaseNotes: releaseNotes,
        run: async (client: Client, docs: any[]): Promise<Profile[]> => {
          return []
        },
      },
    ]
    const result: MigrationResult = {
      type: MIGRATION_RESULT_FAILED,
      errors: [],
    }
    const migrateSpyOneFail = migrateSpy.mockResolvedValueOnce(result)
    mockClient.query.mockResolvedValue(mockQueryResult)

    await ms.runMigrations(migrations)
    expect(migrateSpyOneFail).toBeCalledTimes(2)
  })

  it('should not run migrations with two fail', async () => {
    const schema: Schema = { _id: '1', version: 0 }
    const mockQueryResult: QueryResult<Schema[]> = {
      data: [schema],
      bookmark: '',
      next: false,
      skip: 0,
    }
    const migrations: Migration[] = [
      {
        baseSchemaVersion: 0,
        targetSchemaVersion: 1,
        appVersion: '1.2.4',
        description: 'Removing mailToken from profil',
        docTypes: PROFILE_DOCTYPE,
        releaseNotes: releaseNotes,
        run: async (client: Client, docs: any[]): Promise<Profile[]> => {
          return []
        },
      },
    ]
    const result: MigrationResult = {
      type: MIGRATION_RESULT_FAILED,
      errors: [],
    }
    mockClient.query.mockResolvedValue(mockQueryResult)
    const migrateSpyTwoFailsKo = jest
      .spyOn(Migrate, 'migrate')
      .mockResolvedValueOnce(result)
      .mockResolvedValueOnce(result)
    try {
      await ms.runMigrations(migrations)
      expect(migrateSpyTwoFailsKo).toBeCalledTimes(1)
      expect(migrateSpy).toBeCalledTimes(1)
    } catch (error) {
      expect(error).toEqual(new Error())
    }
  })

  it('should skip migrations if schema number is up to date', async () => {
    const schema: Schema = { _id: '1', version: 1 }
    const mockQueryResult: QueryResult<Schema[]> = {
      data: [schema],
      bookmark: '',
      next: false,
      skip: 0,
    }
    const migrations: Migration[] = [
      {
        baseSchemaVersion: 0,
        targetSchemaVersion: 1,
        appVersion: '1.2.4',
        description: 'Removing mailToken from profil',
        docTypes: PROFILE_DOCTYPE,
        releaseNotes: releaseNotes,
        run: async (client: Client, docs: any[]): Promise<Profile[]> => {
          return []
        },
      },
    ]
    mockClient.query.mockResolvedValue(mockQueryResult)

    await ms.runMigrations(migrations)
    expect(migrateSpy).toBeCalledTimes(0)
  })
  it('should run 2 migrations properly from a fresh instance and dont show releasenotes', async () => {
    const schema: Schema = { _id: '1', version: 0 }
    const mockQueryResult: QueryResult<Schema[]> = {
      data: [schema],
      bookmark: '',
      next: false,
      skip: 0,
    }
    const migrations: Migration[] = [
      {
        baseSchemaVersion: 0,
        targetSchemaVersion: 1,
        appVersion: '1.2.4',
        description: 'Removing mailToken from profil',
        docTypes: PROFILE_DOCTYPE,
        releaseNotes: releaseNotes,
        run: async (client: Client, docs: any[]): Promise<Profile[]> => {
          return []
        },
      },
      {
        baseSchemaVersion: 1,
        targetSchemaVersion: 2,
        appVersion: '1.2.4',
        description: 'Removing mailToken from profil',
        docTypes: PROFILE_DOCTYPE,
        releaseNotes: releaseNotes,
        run: async (client: Client, docs: any[]): Promise<Profile[]> => {
          return []
        },
      },
    ]
    mockClient.query.mockResolvedValue(mockQueryResult)

    const res = await ms.runMigrations(migrations)
    expect(migrateSpy).toBeCalledTimes(2)
    expect(res.show).toBeFalsy()
  })
})
