import { Client } from 'cozy-client'
import { FluidState, FluidType } from 'enum/fluid.enum'
import { DateTime } from 'luxon'
import { Account, FluidStatus, Konnector, Trigger, TriggerState } from 'models'
import { PartnersInfo } from 'models/partnersInfo.model'
import AccountService from 'services/account.service'
import ConsumptionService from 'services/consumption.service'
import ConfigService from 'services/fluidConfig.service'
import KonnectorService from 'services/konnector.service'
import TriggerService from 'services/triggers.service'

export default class FluidService {
  private _client: Client

  constructor(_client: Client) {
    this._client = _client
  }

  private parseFluidStatus = (
    konnector: Konnector | null,
    state: TriggerState | null
  ): FluidState => {
    if (!konnector) return FluidState.KONNECTOR_NOT_FOUND
    if (!state) {
      return FluidState.NOT_CONNECTED
    } else {
      switch (state.status) {
        case 'done':
          return FluidState.DONE
        case 'errored':
          if (state?.last_error === 'LOGIN_FAILED')
            return FluidState.ERROR_LOGIN_FAILED
          else return FluidState.ERROR
        default:
          return FluidState.NOT_CONNECTED
      }
    }
  }

  public parseFluidMaintenance(
    fluidType: FluidType,
    partnersInfo?: PartnersInfo
  ) {
    return Boolean(
      partnersInfo?.notification_activated &&
        ((fluidType === FluidType.GAS && partnersInfo.grdf_failure) ||
          (fluidType === FluidType.ELECTRICITY &&
            partnersInfo.enedis_failure) ||
          (fluidType === FluidType.WATER && partnersInfo.egl_failure))
    )
  }

  public getFluidStatus = async (
    partnersInfo?: PartnersInfo
  ): Promise<FluidStatus[]> => {
    const fluidConfig = new ConfigService().getFluidConfig()
    const accountService = new AccountService(this._client)
    const [elecAccount, waterAccount, gasAccount]: (Account | null)[] =
      await Promise.all([
        accountService.getAccountByType(
          fluidConfig[FluidType.ELECTRICITY].konnectorConfig.slug
        ),
        accountService.getAccountByType(
          fluidConfig[FluidType.WATER].konnectorConfig.slug
        ),
        accountService.getAccountByType(
          fluidConfig[FluidType.GAS].konnectorConfig.slug
        ),
      ])
    const konnectorService = new KonnectorService(this._client)
    const [elecKonnector, waterKonnector, gasKonnector]: (Konnector | null)[] =
      await Promise.all([
        konnectorService.getKonnector(
          fluidConfig[FluidType.ELECTRICITY].konnectorConfig.slug
        ),
        konnectorService.getKonnector(
          fluidConfig[FluidType.WATER].konnectorConfig.slug
        ),
        konnectorService.getKonnector(
          fluidConfig[FluidType.GAS].konnectorConfig.slug
        ),
      ])
    const triggerService = new TriggerService(this._client)
    const [elecTrigger, waterTrigger, gasTrigger]: (Trigger | null)[] =
      await Promise.all([
        elecAccount && elecKonnector
          ? triggerService.getTrigger(elecAccount, elecKonnector)
          : null,
        waterAccount && waterKonnector
          ? triggerService.getTrigger(waterAccount, waterKonnector)
          : null,
        gasAccount && gasKonnector
          ? triggerService.getTrigger(gasAccount, gasKonnector)
          : null,
      ])
    const consumptionService = new ConsumptionService(this._client)
    const [elecStatus, waterStatus, gasStatus]: (TriggerState | null)[] =
      await Promise.all([
        elecTrigger ? triggerService.fetchTriggerState(elecTrigger) : null,
        waterTrigger ? triggerService.fetchTriggerState(waterTrigger) : null,
        gasTrigger ? triggerService.fetchTriggerState(gasTrigger) : null,
      ])
    const firstDataDates: (DateTime | null)[] =
      await consumptionService.fetchAllFirstDateData([
        FluidType.ELECTRICITY,
        FluidType.WATER,
        FluidType.GAS,
      ])
    const lastDataDates: (DateTime | null)[] =
      await consumptionService.fetchAllLastDateData([
        FluidType.ELECTRICITY,
        FluidType.WATER,
        FluidType.GAS,
      ])

    const result: FluidStatus[] = [
      {
        fluidType: FluidType.ELECTRICITY,
        status: this.parseFluidStatus(elecKonnector, elecStatus),
        maintenance: this.parseFluidMaintenance(
          FluidType.ELECTRICITY,
          partnersInfo
        ),
        firstDataDate: firstDataDates[FluidType.ELECTRICITY],
        lastDataDate: lastDataDates[FluidType.ELECTRICITY],
        connection: {
          shouldLaunchKonnector: false,
          isUpdating: false,
          konnector: elecKonnector,
          account: elecAccount,
          trigger: elecTrigger,
          triggerState: elecStatus,
          konnectorConfig: fluidConfig[FluidType.ELECTRICITY].konnectorConfig,
        },
      },
      {
        fluidType: FluidType.WATER,
        status: this.parseFluidStatus(waterKonnector, waterStatus),
        maintenance: this.parseFluidMaintenance(FluidType.WATER, partnersInfo),
        firstDataDate: firstDataDates[FluidType.WATER],
        lastDataDate: lastDataDates[FluidType.WATER],
        connection: {
          shouldLaunchKonnector: false,
          isUpdating: false,
          konnector: waterKonnector,
          account: waterAccount,
          trigger: waterTrigger,
          triggerState: waterStatus,
          konnectorConfig: fluidConfig[FluidType.WATER].konnectorConfig,
        },
      },
      {
        fluidType: FluidType.GAS,
        status: this.parseFluidStatus(gasKonnector, gasStatus),
        maintenance: this.parseFluidMaintenance(FluidType.GAS, partnersInfo),
        firstDataDate: firstDataDates[FluidType.GAS],
        lastDataDate: lastDataDates[FluidType.GAS],
        connection: {
          shouldLaunchKonnector: false,
          isUpdating: false,
          konnector: gasKonnector,
          account: gasAccount,
          trigger: gasTrigger,
          triggerState: gasStatus,
          konnectorConfig: fluidConfig[FluidType.GAS].konnectorConfig,
        },
      },
    ]
    return result
  }

  /**
   * Return fluids with data older than 5 days
   * @param {FluidStatus[]} fluidStatus
   * @returns {FluidType[]}
   */
  static getOldFluidData = async (
    fluidStatus: FluidStatus[]
  ): Promise<FluidType[]> => {
    const fluidOldData: FluidType[] = []
    if (fluidStatus.length > 0) {
      for (const fluid of fluidStatus) {
        let diffInDays = 0
        if (fluid?.lastDataDate) {
          const dateToCompare = fluid.lastDataDate
          diffInDays = dateToCompare.diffNow('days').toObject().days || 0
          if (
            diffInDays < -5 &&
            fluid.status !== FluidState.KONNECTOR_NOT_FOUND &&
            fluid.status !== FluidState.NOT_CONNECTED
          ) {
            !fluidOldData.includes(fluid.fluidType) &&
              fluidOldData.push(fluid.fluidType)
          }
        }
      }
    }
    return fluidOldData
  }
}
