import * as Sentry from '@sentry/react'
import coldWaterData from 'constants/consumptionConstants/coldWater.json'
import cookingData from 'constants/consumptionConstants/cooking.json'
import EcsData from 'constants/consumptionConstants/ecs.json'
import elecSpeData from 'constants/consumptionConstants/electricSpecific.json'
import heatingData from 'constants/consumptionConstants/heating.json'
import { Client } from 'cozy-client'
import logger from 'cozy-logger'
import { FluidType } from 'enum/fluid.enum'
import {
  ConstructionYear,
  Floor,
  HotWaterEquipment,
  HousingType,
  IndividualInsulationWork,
  IndividualOrCollective,
  OutsideFacingWalls,
  ThreeChoicesAnswer,
} from 'enum/profileType.enum'
import { DateTime } from 'luxon'
import {
  DetailsMonthlyForecast,
  FluidForecast,
  MonthlyForecast,
  ProfileType,
} from 'models/profileType.model'
import logApp from 'utils/logger'
import ConverterService from './converter.service'

const logStack = logger.namespace('profileTypeService')

export default class ProfileTypeService {
  private readonly profileType: ProfileType
  private readonly _client: Client
  private readonly year: number

  constructor(profileType: ProfileType, _client: Client, year: number) {
    logApp.info(
      '[ProfileType] Analysis loaded profileType related to: ',
      profileType.updateDate
        ? profileType.updateDate.toString()
        : 'no update date'
    )
    this.profileType = profileType
    this._client = _client
    this.year = year
  }

  /**
   * calculateWarmingEstimatedConsumption
   * @returns {number} Estimated consumption
   */
  public calculateWarmingEstimatedConsumption(): number {
    const ratiosHeating = heatingData.heating_ratio
    const housingType: HousingType = this.profileType.housingType
    const constructionYear: ConstructionYear = this.profileType.constructionYear

    const ratiosHeatingByHousingType = ratiosHeating[housingType]
    const currentRatio: number = ratiosHeatingByHousingType[constructionYear]
    const estimatedConsumption: number =
      parseInt(this.profileType.area) * currentRatio
    return estimatedConsumption
  }

  /**
   * calculateWarmingCorrectedConsumption
   * @param {number} estimatedConsumption
   * @returns {number} - Corrected consumption
   */
  public calculateWarmingCorrectedConsumption(
    estimatedConsumption: number
  ): number {
    const outsideFacingWalls: OutsideFacingWalls =
      this.profileType.outsideFacingWalls
    const housingType: HousingType = this.profileType.housingType
    const floor: Floor = this.profileType.floor
    const constructionYear: ConstructionYear = this.profileType.constructionYear
    const individualInsulationWork: IndividualInsulationWork[] =
      this.profileType.individualInsulationWork
    const hasInstalledVentilation: ThreeChoicesAnswer =
      this.profileType.hasInstalledVentilation
    const hasReplacedHeater: ThreeChoicesAnswer =
      this.profileType.hasReplacedHeater
    const heating = this.profileType.heating

    // Apply corrections
    const correctionsNbWalls =
      heatingData.adjustment_outside_facing_walls[housingType]
    const correctionWalls: number = correctionsNbWalls[outsideFacingWalls]

    let correctionFloor: number
    let correctionInsulation: number
    let correctionFacilities: number

    if (this.profileType.housingType === HousingType.APARTMENT) {
      correctionFloor = heatingData.adjustment_floor.apartment[floor]
    } else correctionFloor = 0

    if (
      !individualInsulationWork.includes(IndividualInsulationWork.NONE) &&
      constructionYear !== ConstructionYear.AFTER_1998 &&
      heating !== IndividualOrCollective.COLLECTIVE
    ) {
      const correctionsInsulation: { [key: string]: number } =
        heatingData.adjustment_insulation[constructionYear]
      if (individualInsulationWork.length === 1) {
        correctionInsulation =
          correctionsInsulation[individualInsulationWork[0]]
      } else {
        let key = ''
        if (
          individualInsulationWork.includes(
            IndividualInsulationWork.ROOF_INSULATION
          ) &&
          individualInsulationWork.includes(
            IndividualInsulationWork.WINDOW_REPLACEMENT
          ) &&
          individualInsulationWork.includes(
            IndividualInsulationWork.WALL_INSULATION
          )
        ) {
          key = 'window_replacement_and_roof_and_wall_insulation'
        } else if (
          individualInsulationWork.includes(
            IndividualInsulationWork.ROOF_INSULATION
          ) &&
          individualInsulationWork.includes(
            IndividualInsulationWork.WINDOW_REPLACEMENT
          )
        ) {
          key = 'window_replacement_and_roof_insulation'
        } else if (
          individualInsulationWork.includes(
            IndividualInsulationWork.ROOF_INSULATION
          ) &&
          individualInsulationWork.includes(
            IndividualInsulationWork.WALL_INSULATION
          )
        ) {
          key = 'roof_and_wall_insulation'
        } else if (
          individualInsulationWork.includes(
            IndividualInsulationWork.WINDOW_REPLACEMENT
          ) &&
          individualInsulationWork.includes(
            IndividualInsulationWork.WALL_INSULATION
          )
        ) {
          key = 'window_replacement_and_wall_insulation'
        }
        correctionInsulation = correctionsInsulation[key]
      }
    } else correctionInsulation = 0

    if (
      hasInstalledVentilation === ThreeChoicesAnswer.YES &&
      hasReplacedHeater === ThreeChoicesAnswer.YES &&
      heating !== IndividualOrCollective.COLLECTIVE
    ) {
      correctionFacilities =
        heatingData.adjustment_facilities['individual_heater_and_ventilation']
    } else if (hasReplacedHeater === ThreeChoicesAnswer.YES) {
      correctionFacilities =
        heatingData.adjustment_facilities['individual_heater']
    } else if (hasInstalledVentilation === ThreeChoicesAnswer.YES) {
      correctionFacilities =
        heatingData.adjustment_facilities['individual_ventilation']
    } else {
      correctionFacilities = 0
    }

    const correctedConsumption =
      estimatedConsumption *
      (1 + correctionFloor) *
      (1 + correctionWalls) *
      (1 + correctionInsulation) *
      (1 + correctionFacilities)

    return Math.round(correctedConsumption)
  }

  /**
   * calculateWarmingMonthConsumption
    @param {number} correctedConsumption
    @param {number} number
    @returns {number} monthConsumption
 */
  public async calculateWarmingMonthConsumption(
    correctedConsumption: number,
    month: number
  ): Promise<number> {
    const djuCurrentMonth = await this.fetchDJU(month)
    const monthConsumption =
      (correctedConsumption / heatingData.dju_annual) * djuCurrentMonth

    return Math.round(monthConsumption)
  }
  /**
   * getMonthHeating
   * @param {ProfileType} profileType
   * @param {number} month
   * @returns {number} Month heating consumption in kw/h
   */
  public async getMonthHeating(month: number): Promise<number> {
    const estimatedConsumption = this.calculateWarmingEstimatedConsumption()
    const correctedConsumption =
      this.calculateWarmingCorrectedConsumption(estimatedConsumption)
    const monthConsumption = await this.calculateWarmingMonthConsumption(
      correctedConsumption,
      month
    )
    return monthConsumption
  }

  /**
   * Calculate water raw needs by month
   *
   * @param {ProfileType} profileType
   * @param {number} month
   * @returns {number} rawConsumption
   */
  public calculateMonthWaterRawNeeds(
    profileType: ProfileType,
    month: number
  ): number {
    const waterNeedsIndex: number = EcsData.ecs_ratio.findIndex(
      ratio => ratio.occupants_number === profileType.occupantsNumber
    )
    const waterNeeds: number =
      EcsData.ecs_ratio[waterNeedsIndex].warm_water_need_in_liter
    const nbDaysInMonth: number = DateTime.fromObject({
      month: month,
    }).setZone('utc', {
      keepLocalTime: true,
    }).daysInMonth
    const rawConsumption: number = waterNeeds * nbDaysInMonth
    return rawConsumption
  }
  /**
   * Calculate spread water needs by month
   *
   * @param {ProfileType} profileType
   * @param {number} month
   * @returns {number} spreadConsumption
   */
  public calculateSpreadNeeds(profileType: ProfileType, month: number): number {
    const rawNeeds: number[] = []
    for (let index = 1; index < 13; index++) {
      rawNeeds.push(this.calculateMonthWaterRawNeeds(profileType, index))
    }
    const calculateTotalNeeds = (
      accumulator: number,
      currentValue: number
    ): number => accumulator + currentValue
    const totalRawNeeds: number = rawNeeds.reduce(calculateTotalNeeds)
    const spreadConsumption: number =
      (totalRawNeeds / (12 * 100)) * EcsData.mensual_repartition[month - 1]
    return Math.round(spreadConsumption)
  }
  /**
   * Calculate total ecs consumption by month
   *
   * @param {number} spreadConsumption
   * @param {ProfileType} profileType
   * @returns {number} monthEcsConsumption
   */
  public calculateTotalConsumption(
    spreadConsumption: number,
    profileType: ProfileType,
    month: number
  ): number {
    const coldWaterTemperature: number =
      EcsData.cold_water_temperature[month - 1]
    const efficiencyProduction: number =
      EcsData.efficiency_production_distribution[profileType.housingType]
    const solarCoverage: number = EcsData.solar_coverage[month - 1]
    const ecsConsumption: number =
      (((EcsData.coefficient_ecs_consumption * spreadConsumption) / 1000) *
        (EcsData.hot_water_temperature - coldWaterTemperature)) /
      efficiencyProduction
    let monthEcsConsumption: number
    if (profileType.hotWaterEquipment === HotWaterEquipment.SOLAR) {
      monthEcsConsumption = ecsConsumption * (1 - solarCoverage)
    } else if (
      profileType.hotWaterEquipment === HotWaterEquipment.THERMODYNAMIC
    ) {
      monthEcsConsumption = ecsConsumption / EcsData.coefficient_thermodynamic
    } else {
      monthEcsConsumption = ecsConsumption
    }
    return Math.round(monthEcsConsumption)
  }
  /**
   * Get ECS consumption by month
   *
   * @param {ProfileType} profileType
   * @param {number} month
   * @returns {number} monthEcsConsumption
   */
  public getMonthEcs(month: number) {
    const spreadConsumption: number = this.calculateSpreadNeeds(
      this.profileType,
      month
    )
    const monthEcsConsumption: number = this.calculateTotalConsumption(
      spreadConsumption,
      this.profileType,
      month
    )
    return monthEcsConsumption
  }
  /**
   * Get cooking consumption by month
   * @param {ProfileType} profileType
   * @param {number} month
   * @returns {number} monthCookingConsumption
   */
  public getMonthCookingConsumption(month: number): number {
    const annualCookingConsumption: number =
      this.profileType.occupantsNumber *
      cookingData.ratio_kw_per_person_per_year
    const nbDaysInMonth: number = DateTime.fromObject({
      month: month,
    }).setZone('utc', {
      keepLocalTime: true,
    }).daysInMonth
    const monthCookingConsumption: number =
      (annualCookingConsumption / 365) * nbDaysInMonth
    return Math.round(monthCookingConsumption)
  }
  /**
   * Get specific electricity consumption by month
   * @param {ProfileType} profileType
   * @param {number} month
   * @returns {number} monthElectricSpecificConsumption
   */
  public getMonthElectricSpecificConsumption(month: number): number {
    const annualElectricSpecificConsumption: number =
      elecSpeData[this.profileType.housingType][
        this.profileType.constructionYear
      ]
    const nbDaysInMonth: number = DateTime.fromObject({
      month: month,
    }).setZone('utc', {
      keepLocalTime: true,
    }).daysInMonth
    const monthElectricSpecificConsumption: number =
      (annualElectricSpecificConsumption / 365) * nbDaysInMonth
    return Math.round(monthElectricSpecificConsumption)
  }
  /**
   * Get cold water consumption by month
   * @param {ProfileType} profileType
   * @param {number} month
   * @returns {number} monthColdWaterConsumption
   */
  public getMonthColdWaterConsumption(month: number): number {
    const occupantsNumberIndex: number = coldWaterData.findIndex(
      waterNeeds =>
        waterNeeds.occupants_number === this.profileType.occupantsNumber
    )
    const coldWaterNeeds: number =
      coldWaterData[occupantsNumberIndex].consumption_in_liter_per_day
    const nbDaysInMonth: number = DateTime.fromObject({
      month: month,
    }).setZone('utc', {
      keepLocalTime: true,
    }).daysInMonth
    const monthColdWaterConsumption: number = nbDaysInMonth * coldWaterNeeds
    return Math.round(monthColdWaterConsumption)
  }
  /**
   * getDetailsMonthlyForecast
   * @param {FluidType} fluidType
   * @param {number} month
   * @returns {number} DetailsMonthlyForecast
   */
  public async getDetailsMonthlyForecast(
    fluidType: FluidType,
    month: number
  ): Promise<DetailsMonthlyForecast> {
    const warmingFluid = this.profileType.warmingFluid
    const hotWaterFluid = this.profileType.hotWaterFluid
    const cookingFluid = this.profileType.cookingFluid

    const detailsMonthlyForecast = {
      heatingConsumption:
        this.profileType.heating === IndividualOrCollective.COLLECTIVE
          ? null
          : warmingFluid !== null &&
            (fluidType as number) === (warmingFluid as number)
          ? await this.getMonthHeating(month)
          : null,
      ecsConsumption:
        this.profileType.heating === IndividualOrCollective.COLLECTIVE
          ? null
          : hotWaterFluid !== null &&
            (fluidType as number) === (hotWaterFluid as number)
          ? this.getMonthEcs(month)
          : null,
      cookingConsumption:
        fluidType === cookingFluid
          ? this.getMonthCookingConsumption(month)
          : null,
      electricSpecificConsumption:
        fluidType === FluidType.ELECTRICITY
          ? this.getMonthElectricSpecificConsumption(month)
          : null,
      coldWaterConsumption:
        fluidType === FluidType.WATER
          ? this.getMonthColdWaterConsumption(month)
          : null,
    }
    return detailsMonthlyForecast
  }

  /**
    * getFluidForecast
    * @param {FluidType} fluidType
    * @param {number} month
    @returns {FluidForecast} fluidForecast
    */
  public async getFluidForecast(
    fluidType: FluidType,
    month: number
  ): Promise<FluidForecast> {
    const detailsMonthlyForecast: DetailsMonthlyForecast =
      await this.getDetailsMonthlyForecast(fluidType, month)
    let fluidLoad = 0

    Object.values(detailsMonthlyForecast).forEach(load => {
      if (load !== null) fluidLoad += load
    })

    const converterService = new ConverterService()
    const fluidValue = parseFloat(
      converterService.LoadToEuro(fluidLoad, fluidType).toPrecision(5)
    )

    const fluidForecast: FluidForecast = {
      fluidType: fluidType,
      load: fluidLoad,
      value: fluidValue,
      detailsMonthlyForecast: detailsMonthlyForecast,
    }
    return fluidForecast
  }

  /**
   * getMonthlyForecast
   * @param {number} month
   * @returns {MonthlyForecast} MonthlyForecast
   */
  public async getMonthlyForecast(month: number): Promise<MonthlyForecast> {
    const elecForecast: FluidForecast = await this.getFluidForecast(
      FluidType.ELECTRICITY,
      month
    )
    const waterForecast: FluidForecast = await this.getFluidForecast(
      FluidType.WATER,
      month
    )
    const gasForecast: FluidForecast = await this.getFluidForecast(
      FluidType.GAS,
      month
    )

    const monthlyForecast = {
      month: month,
      fluidForecast: [elecForecast, waterForecast, gasForecast],
      totalValue: elecForecast.value + waterForecast.value + gasForecast.value,
    }
    return monthlyForecast
  }

  /**
   * Try to fetch dju from remote doctype, if no data or error, return default data
   * @param {Client} client
   * @param {number} month
   * @returns {number} monthDju
   */
  public fetchDJU = async (month: number): Promise<number> => {
    const startDate: string = DateTime.local()
      .setZone('utc', {
        keepLocalTime: true,
      })
      .set({ year: this.year })
      .set({ month: month })
      .startOf('month')
      .minus({ minutes: 5 })
      .toISO()
    const endDate: string = DateTime.local()
      .setZone('utc', {
        keepLocalTime: true,
      })
      .set({ year: this.year })
      .set({ month: month })
      .endOf('month')
      .plus({ minutes: 5 })
      .toISO()
    // We add or remove 5 minutes so the api returns the exact period
    try {
      const result = await this._client
        .getStackClient()
        .fetchJSON(
          'GET',
          `/remote/org.ecolyo.dju?observedProperty=degreeDay&startDate=${startDate}&endDate=${endDate}`
        )
      let monthDju = 0

      if (result) {
        for (const observation of result.observations) {
          monthDju += observation.result.value
        }
      }
      if (monthDju === 0) {
        return heatingData.dju_average_by_month[month - 1]
      } else {
        return monthDju
      }
    } catch (error) {
      const errorMessage = `fetchDju error : ${JSON.stringify(error)}`
      logStack('error', errorMessage)
      logApp.error(errorMessage)
      Sentry.captureException(errorMessage)
      return heatingData.dju_average_by_month[month - 1]
    }
  }

  /**
   * checkConsistency
   * @param {ProfileType} profileType
   * @returns {ProfileType} consistent profile type
   */
  static checkConsistency(profileType: ProfileType): ProfileType {
    if (profileType.housingType === HousingType.INDIVIDUAL_HOUSE) {
      profileType.floor = Floor.NOT_APPLICABLE
      profileType.heating = IndividualOrCollective.INDIVIDUAL
      profileType.coldWater = IndividualOrCollective.INDIVIDUAL
      profileType.hotWater = IndividualOrCollective.INDIVIDUAL
    } else {
      if (profileType.heating === IndividualOrCollective.COLLECTIVE) {
        profileType.warmingFluid = null
        profileType.individualInsulationWork = []
        profileType.facilitiesInstallation = []
      }
      if (profileType.hotWater === IndividualOrCollective.COLLECTIVE) {
        profileType.hotWaterFluid = null
        profileType.hotWaterEquipment = HotWaterEquipment.OTHER
      }
    }
    if (profileType.constructionYear === ConstructionYear.AFTER_1998) {
      profileType.individualInsulationWork = [IndividualInsulationWork.NONE]
    }
    // Remove potential metadata from already existing profileType in order
    // to create a new entry. Otherwise will update existing one.
    if (profileType._id) {
      delete profileType._id
      delete profileType._rev
      delete profileType._type
      delete profileType.cozyMetadata
    }
    if (profileType.updateDate) {
      profileType.updateDate = profileType.updateDate.setZone('utc', {
        keepLocalTime: true,
      })
    }
    return profileType
  }
}
