import * as Sentry from '@sentry/react'
import { Client, Q, QueryDefinition, QueryResult } from 'cozy-client'
import logger from 'cozy-logger'
import profileTypeData from 'db/profileTypeData.json'
import { PROFILETYPE_DOCTYPE } from 'doctypes'
import { DateTime } from 'luxon'
import { ProfileType, TimePeriod } from 'models'
import logApp from 'utils/logger'

const logStack = logger.namespace('profileTypeEntityService')
export default class ProfileTypeEntityService {
  private readonly _client: Client

  constructor(_client: Client) {
    this._client = _client
  }

  /**
   * Retrieves ProfileType from the PROFILETYPE_DOCTYPE
   * When called with date parameter, fetches closest profileType to the date
   * When called without parameters, fetches last profileType in doctype
   * @param {DateTime} date
   * @returns {ProfileType}
   */
  public async getProfileType(date?: DateTime): Promise<ProfileType | null> {
    const query: QueryDefinition = Q(PROFILETYPE_DOCTYPE)
    if (date) {
      const {
        data: [profileType],
      }: QueryResult<ProfileType[]> = await this._client.query(
        query
          .where({ updateDate: { $lte: date.toString() } })
          .sortBy([{ updateDate: 'desc' }])
          .limitBy(1)
      )
      const result: ProfileType | null = profileType ? profileType : null
      if (result) {
        return this.parseProfileTypeEntityToProfileType(profileType)
      } else {
        logApp.debug('No profileType found for: ', date.toString())
        logApp.debug(
          'Checking if user has already filled a profileType and uses it as default'
        )
        const query: QueryDefinition = Q(PROFILETYPE_DOCTYPE)
        const data: QueryResult<ProfileType[]> = await this._client.query(query)
        if (data.data.length) {
          const loadedProfileType: ProfileType = data.data[0]
          logApp.debug(
            'found oldest profileType filled by user : ',
            loadedProfileType
          )
          return loadedProfileType
        } else {
          // return default profiletype
          const loadedProfileType: any = {
            ...profileTypeData[0].profileType,
            updateDate: date,
          }
          logApp.debug('No profileType were found, loading default profileType')
          return loadedProfileType
        }
      }
    } else {
      const {
        data: [profileType],
      }: QueryResult<ProfileType[]> = await this._client.query(
        query
          .where({ _id: { $gt: null } })
          .sortBy([{ updateDate: 'desc' }])
          .limitBy(1)
      )
      const result: ProfileType | null = profileType ? profileType : null
      if (result) {
        return this.parseProfileTypeEntityToProfileType(profileType)
      }
    }
    return null
  }

  /**
   * Retrieves all ProfileTypes from the PROFILETYPE_DOCTYPE
   * When called with period parameter, returns all profileTypes found for this period or null
   * When called without parameters, returns all profileTypes or null if none exist
   * @param {TimePeriod} timePeriod
   * @returns {ProfileType}
   */
  public async getAllProfileTypes(
    timePeriod?: TimePeriod
  ): Promise<ProfileType[] | null> {
    const query: QueryDefinition = Q(PROFILETYPE_DOCTYPE)
    if (timePeriod) {
      const data: QueryResult<ProfileType[]> = await this._client.query(
        query
          .where({
            updateDate: {
              $gte: timePeriod.startDate.toString(),
            },
          })
          .sortBy([{ updateDate: 'asc' }])
          .limitBy(100)
      )
      if (data.data.length) {
        const profileTypesToReturn: ProfileType[] = []
        data.data.forEach((ele: ProfileType) => {
          profileTypesToReturn.push(
            this.parseProfileTypeEntityToProfileType(ele)
          )
        })
        return profileTypesToReturn
      } else {
        return null
      }
    } else {
      const data: QueryResult<ProfileType[]> = await this._client.query(
        query.where({ _id: { $gt: null } }).sortBy([{ updateDate: 'desc' }])
      )
      if (data.data.length) {
        const profileTypesToReturn: ProfileType[] = []
        data.data.forEach((ele: ProfileType) => {
          profileTypesToReturn.push(
            this.parseProfileTypeEntityToProfileType(ele)
          )
        })
        return profileTypesToReturn
      }
    }
    return null
  }

  /**
   * Saves profileType in database
   * @returns {ProfileType}
   */
  public async updateProfileType(
    attributes: Partial<ProfileType>
  ): Promise<ProfileType | null> {
    const query: QueryDefinition = Q(PROFILETYPE_DOCTYPE)
    const {
      data: [doc],
    }: QueryResult<ProfileType[]> = await this._client.query(query.limitBy(1))
    const { data: profileTypeEntity }: QueryResult<ProfileType | null> =
      await this._client.save({
        ...doc,
        ...attributes,
      })
    if (profileTypeEntity) {
      return this.parseProfileTypeEntityToProfileType(profileTypeEntity)
    }
    return null
  }

  /**
   * Deletes Array of ProfileTypes
   * @returns {boolean}
   */
  public async deleteProfileTypes(
    profileTypes: ProfileType[]
  ): Promise<boolean> {
    try {
      for (let index = 0; index < profileTypes.length; index++) {
        await this._client.destroy(profileTypes[index])
      }
      return true
    } catch (error) {
      const errorMessage = `deleteProfileTypes: ${JSON.stringify(error)}`
      logStack('error', errorMessage)
      logApp.error(errorMessage)
      Sentry.captureException(errorMessage)
      return false
    }
  }

  /**
   * Retrieves ProfileType from the ProfileTypeEntity
   * @param {ProfileType} profileTypeEntity
   * @returns {ProfileType}
   */
  private parseProfileTypeEntityToProfileType(
    profileTypeEntity: ProfileType
  ): ProfileType {
    const profileType: ProfileType = {
      ...profileTypeEntity,
      updateDate:
        typeof profileTypeEntity.updateDate === 'string'
          ? DateTime.fromISO(profileTypeEntity.updateDate, {
              zone: 'utc',
            })
          : profileTypeEntity.updateDate,
    }
    return profileType
  }
}
