import { QueryResult } from 'cozy-client'
import { DateTime } from 'luxon'
import { Term } from 'models'
import mockClient from '../../tests/__mocks__/client'
import mockConfig from '../../tests/__mocks__/mockConfig.mock'
import {
  mockOutdatedTerm,
  mockTermsData,
  mockUpToDateTerm,
} from '../../tests/__mocks__/termsData.mock'
import TermsService from './terms.service'

const localSpy = jest.spyOn(DateTime, 'local')

const termsService = new TermsService(mockClient)

jest.mock('../constants/config.json', () => mockConfig, {
  virtual: true,
})
describe('TermsService service', () => {
  it('should get the last term', async () => {
    const mockQueryResult: QueryResult<Term[]> = {
      data: [mockTermsData[0]],
      bookmark: '',
      next: false,
      skip: 0,
    }
    mockClient.query.mockResolvedValueOnce(mockQueryResult)
    const result = await termsService.getLastTerm()
    expect(result).toEqual(mockTermsData[0])
  })
  it('should compare version and return an outdated case', async () => {
    const mockQueryResult: QueryResult<Term[]> = {
      data: [mockOutdatedTerm],
      bookmark: '',
      next: false,
      skip: 0,
    }
    mockClient.query.mockResolvedValueOnce(mockQueryResult)
    const result = await termsService.isConsentVersionUpToDate()
    expect(result).toEqual(false)
  })
  it('should compare version and return an up-to-date case', async () => {
    const mockQueryResult: QueryResult<Term[]> = {
      data: [mockUpToDateTerm],
      bookmark: '',
      next: false,
      skip: 0,
    }
    mockClient.query.mockResolvedValueOnce(mockQueryResult)
    const result = await termsService.isConsentVersionUpToDate()
    expect(result).toEqual(true)
  })
  it('should return true when the last term has been accepted by user', async () => {
    const mockQueryResult: QueryResult<Term[]> = {
      data: [mockUpToDateTerm],
      bookmark: '',
      next: false,
      skip: 0,
    }
    mockClient.query.mockResolvedValueOnce(mockQueryResult)
    const result = await termsService.isLastTermValidated()
    expect(result).toEqual(true)
  })
  it('should return false when the last term has not been accepted by user', async () => {
    mockUpToDateTerm.accepted = false
    const mockQueryResult: QueryResult<Term[]> = {
      data: [mockUpToDateTerm],
      bookmark: '',
      next: false,
      skip: 0,
    }
    mockClient.query.mockResolvedValueOnce(mockQueryResult)
    const result = await termsService.isLastTermValidated()
    expect(result).toEqual(false)
  })
  it('should return a major update', async () => {
    mockUpToDateTerm.accepted = false
    const mockQueryResult: QueryResult<Term[]> = {
      data: [mockUpToDateTerm],
      bookmark: '',
      next: false,
      skip: 0,
    }
    mockClient.query.mockResolvedValueOnce(mockQueryResult)
    const result = await termsService.getTermsVersionType()
    expect(result).toEqual('major')
  })
  it('should return a minor update', async () => {
    mockUpToDateTerm.accepted = false
    const mockQueryResult: QueryResult<Term[]> = {
      data: [mockTermsData[1]],
      bookmark: '',
      next: false,
      skip: 0,
    }
    mockClient.query.mockResolvedValueOnce(mockQueryResult)
    const result = await termsService.getTermsVersionType()
    expect(result).toEqual('minor')
  })
  it('should create a new term', async () => {
    const mockDate = DateTime.fromISO(mockUpToDateTerm.acceptedAt.toString(), {
      zone: 'utc',
    })
    localSpy.mockReturnValueOnce(mockDate)
    const mockQueryResult: QueryResult<Term> = {
      data: mockUpToDateTerm,
      bookmark: '',
      next: false,
      skip: 0,
    }
    mockClient.create.mockResolvedValueOnce(mockQueryResult)
    const result = await termsService.createTerm()
    expect(result).toEqual(mockUpToDateTerm)
  })
  it('should fail create a new term', async () => {
    mockClient.create.mockRejectedValue(new Error())
    await expect(termsService.createTerm()).rejects.toThrow(new Error())
  })
})
